{ There are two possible ways to work with the library Gipals32.dll.

 1) First way is a linear program construction by the provided set of functions,
    such as specification of the variables and constraints count and assignment
    of the constraint matrix elements. This way provides a complete flexibility
    of the linear programming modeling.

 2) The second way is importing linear program from a text file in MPS format.
    It can be done by one function LoadFromMPS. This way can be suitable if there
    is a set of prepared MPS files that need to be calculated.

  The example program work as following:
  If there is a command line parameter then it's considered as MPS file and the
  program tries to load and calculate it.
  If there are no command line parameters then internal example of a transport
  problem is loaded and calculated.

}

program TestGipalsDelphi;
{$APPTYPE CONSOLE}
uses
  HeaderGipalsDelphi,
  SysUtils;

{
  This is an example of a transport problem (Minimal Cost).

  There are three warehouses at different cities: Detroit, Pittsburgh and Buffalo.
  They have 250, 130 and 235 tons of paper accordingly. There are four publishers in
  Boston, New York, Chicago and Indianapolis. They ordered 75, 230, 240 and 70 tons of
  paper to publish new books.
  There are the following costs in dollars of transportation of one ton of paper:

  From \ To	Boston (BS)	New York (NY)	Chicago (CH)	Indianapolis (IN)
  Detroit (DT)	         15                20             16                   21
  Pittsburgh (PT)        25                13              5                   11
  Buffalo (BF)           15                15              7                   17

  We denote the cost of transportation from one city to another by C_ prefix, for example
  cost from Buffalo to Chicago is C_BF_CH.

  We have to find a plan that all orders will be performed and the transportation costs
  will be minimized. The decision variables are amounts of tons transported from one
  city to another denoted by prefix X_:

  X_DT_BS = tons from Detroit to Boston,
  X_PT_BS = tons from Pittsburgh to Boston,
  X_BF_NY = tons from Buffalo to New York and etc.

  The objective function is sum of multiplications of transported tons and their costs:
  F = C_DT_BS * X_DT_BS + C_DT_NY * C_DT_NY + C_DT_CH * C_DT_CH + 

  The constraints are defined in terms of the transported paper amount from one
  warehouse or to one publisher.

  For example:
  Amount transported from Detroit cannot be more than 250 tons:
  X_DT_BS + X_DT_NY + X_DT_CH + X_DT_IN <= 250
}

Procedure FillVariables;
begin
  AddVariable(15, 0, 0, 0, 1); // Tons from Detroit to Boston. Cost per tone is $15
  AddVariable(20, 0, 0, 0, 1); // Tons from Detroit to New York. Cost per tone is $20
  AddVariable(16, 0, 0, 0, 1); // Tons from Detroit to Chicago. Cost per tone is $16
  AddVariable(21, 0, 0, 0, 1); // Tons from Detroit to Indianapolis. Cost per tone is $21
  AddVariable(25, 0, 0, 0, 1); // Tons from Pittsburgh to Boston. Cost per tone is $25
  AddVariable(13, 0, 0, 0, 1); // Tons from Pittsburgh to New York. Cost per tone is $13
  AddVariable( 5, 0, 0, 0, 1); // Tons from Pittsburgh to Chicago. Cost per tone is $5
  AddVariable(11, 0, 0, 0, 1); // Tons from Pittsburgh to Indianapolis. Cost per tone is $11
  AddVariable(15, 0, 0, 0, 1); // Tons from Buffalo to Boston. Cost per tone is $15
  AddVariable(15, 0, 0, 0, 1); // Tons from Buffalo to New York. Cost per tone is $15
  AddVariable( 7, 0, 0, 0, 1); // Tons from Buffalo to Chicago. Cost per tone is $7
  AddVariable(17, 0, 0, 0, 1); // Tons from Buffalo to Indianapolis.Cost per tone is $17
end;

Procedure FillConstraints;
begin
  AddConstraint(250, sign_Less);  // Transported from Detroit must be less then 250 tones.
  AddConstraint(130, sign_Less);  // Transported from Pittsburgh must be less then 130.
  AddConstraint(235, sign_Less);  // Transported from Buffalo must be less then 235.
  AddConstraint( 75, sign_Equal); // Shipped to Boston must be 75 tones.
  AddConstraint(230, sign_Equal); // Shipped to New York must be 230 tones.
  AddConstraint(240, sign_Equal); // Shipped to Chicago must be 240 tones.
  AddConstraint( 70, sign_Equal); // Shipped to Indianapolis must be 70 tones.
end;

Procedure FillMatrix;
begin
  // X_DT_BS + X_DT_NY + X_DT_CH + X_DT_IN <= 250
  DirectSelectConstraint(0);
  DirectAddConstraintElement(0, 1); // X_DT_BS
  DirectAddConstraintElement(1, 1); // X_DT_NY
  DirectAddConstraintElement(2, 1); // X_DT_CH
  DirectAddConstraintElement(3, 1); // X_DT_IN

  // X_PT_BS + X_PT_NY + X_PT_CH + X_PT_IN <= 130
  DirectSelectConstraint(1);
  DirectAddConstraintElement(4, 1); // X_PT_BS
  DirectAddConstraintElement(5, 1); // X_PT_NY
  DirectAddConstraintElement(6, 1); // X_PT_CH
  DirectAddConstraintElement(7, 1); // X_PT_IN

  // X_BF_BS + X_BF_NY + X_BF_CH + X_BF_IN <= 235
  DirectSelectConstraint(2);
  DirectAddConstraintElement(8, 1); // X_BF_BS
  DirectAddConstraintElement(9, 1); // X_BF_NY
  DirectAddConstraintElement(10, 1); // X_BF_CH
  DirectAddConstraintElement(11, 1); // X_BF_IN

  // X_DT_BS + X_PT_BS + X_BF_BS = 75
  DirectSelectConstraint(3);
  DirectAddConstraintElement(0, 1); // X_DT_BS
  DirectAddConstraintElement(4, 1); // X_PT_BS
  DirectAddConstraintElement(8, 1); // X_BF_BS

  // X_DT_NY + X_PT_NY + X_BF_NY = 230
  DirectSelectConstraint(4);
  DirectAddConstraintElement(1, 1); // X_DT_NY
  DirectAddConstraintElement(5, 1); // X_PT_NY
  DirectAddConstraintElement(9, 1); // X_BF_NY

  // X_DT_CH + X_PT_CH + X_BF_CH = 240
  DirectSelectConstraint(5);
  DirectAddConstraintElement( 2, 1); // X_DT_CH
  DirectAddConstraintElement( 6, 1); // X_PT_CH
  DirectAddConstraintElement(10, 1); // X_BF_CH

  // X_DT_IN + X_PT_IN + X_BF_IN = 70
  DirectSelectConstraint(6);
  DirectAddConstraintElement( 3, 1); // X_DT_IN
  DirectAddConstraintElement( 7, 1); // X_PT_IN
  DirectAddConstraintElement(11, 1); // X_BF_IN
end;

Var
  Iteration,Iter: Integer; // Current iteration
  FName: String;
  IsTransport: Boolean; // Indicates transport problem

Function ProcessInfo(IterCount: Integer;
                     PrimalObj,
                     DualObj,
                     InfPrimal,
                     InfDual,
                     Optimality: Double): Integer; stdcall;
begin
  Result := res_None;
  if Iteration <> IterCount then
  begin
    Writeln(Format(' %4d | %11.6g | %11.6g | %11.6g | %11.6g | %11.6g',
                   [IterCount, PrimalObj, DualObj, InfPrimal, InfDual, Optimality]));
    Iteration := IterCount;
  end;
end;

begin
  {if the parameter is specified then assume it's MPS file}
  IsTransport := ParamCount = 0;
  if not IsTransport then
  begin
    FName := ExtractFilePath(ParamStr(0)) + ParamStr(1);
    Writeln('Importing linear program from file: ', FName, '...');
    LoadFromMPS(PChar(FName));
    Writeln('Import is done');
  end
  else
  begin
    Writeln('This is an example of a transport problem (Minimal Cost)');
    FillVariables;
    FillConstraints;
    FillMatrix;
  end;
  Writeln('---------------------------------------------------------------------------');
  Writeln(' No.  |   Objective Functions     |        Infeasibility      | Optimality  ');
  Writeln(' Iter.|   Primal    |    Dual     |   Primal    |     Dual    |             ');
  Writeln('---------------------------------------------------------------------------');
  Case Calculate(@ProcessInfo) of
           calc_None: Writeln('Calculation has not been provided yet');
        calc_Optimal: Writeln('Finished with optimal solution');
     calc_Infeasible: Writeln('Failed with primal infeasibility');
  calc_DblInfeasible: Writeln('Failed with dual infeasibility');
        calc_Unknown: Writeln('Finished because reached the max. of iterations');
     calc_Suboptimal: Writeln('Finished due to slow convergence');
         calc_Stoped: Writeln('Stoped by the user');
          calc_Error: Writeln('Failed due to numerical eror');
      calc_Unbounded: Writeln('Failed because the linear program is unbounded');
  end;
  Writeln('');
  if IsTransport then
  begin
    Writeln('The Total Cost of Transportation is $', PrimalObjective:10:2);
    Writeln('');
    // output results of the calculation
    Writeln('Shipment plan:');
    Writeln('');
    Writeln(PrimalValue(0):10:2,  ' Tons from Detroit to Boston');
    Writeln(PrimalValue(1):10:2,  ' Tons from Detroit to New York');
    Writeln(PrimalValue(2):10:2,  ' Tons from Detroit to Chicago');
    Writeln(PrimalValue(3):10:2,  ' Tons from Detroit to Indianapolis');
    Writeln(PrimalValue(4):10:2,  ' Tons from Pittsburgh to Boston');
    Writeln(PrimalValue(5):10:2,  ' Tons from Pittsburgh to New York');
    Writeln(PrimalValue(6):10:2,  ' Tons from Pittsburgh to Chicago');
    Writeln(PrimalValue(7):10:2,  ' Tons from Pittsburgh to Indianapolis');
    Writeln(PrimalValue(8):10:2,  ' Tons from Buffalo to Boston');
    Writeln(PrimalValue(9):10:2,  ' Tons from Buffalo to New York');
    Writeln(PrimalValue(10):10:2, ' Tons from Buffalo to Chicago');
    Writeln(PrimalValue(11):10:2, ' Tons from Buffalo to Indianapolis');
  end
  else
  begin
    Writeln('Primal Objective: ', PrimalObjective);
    Writeln('  Dual Objective: ', DualObjective);
    Writeln('');
    // output results of the calculation
    Writeln('Primal Results.');
    Writeln('   #    Variable (X)      Reduced Cost (S)');
    for iter := 0 to GetVariableCount - 1 do
    begin
      Writeln(Format('%4d    %12.6g          %12.6g', [iter, PrimalValue(iter), ReducedCostValue(iter)]));
    end;
    Writeln('');
    Writeln('Dual Results.');
    Writeln('   #    Dual Variable (Y)');
    for iter := 0 to GetConstraintCount - 1 do
    begin
      Writeln(Format('%4d    %12.6g', [iter, DualValue(iter)]));
    end;
  end;
  Writeln('');
  Writeln('Press Enter...');
  Readln;
end.
